-Wflex-array-member-not-at-end is coming in GCC-14, and we are getting
ready to enable it globally.

Use the `__struct_group()` helper to separate the flexible array
from the rest of the members in flexible `struct nx842_crypto_header`,
through tagged `struct nx842_crypto_header_hdr`, and avoid embedding
the flexible-array member in the middle of `struct nx842_crypto_ctx`.

Also, use `container_of()` whenever we need to retrieve a pointer to
the flexible structure.

This code was detected with the help of Coccinelle, and audited and
modified manually.

Link: https://github.com/KSPP/linux/issues/202
Signed-off-by: Gustavo A. R. Silva <gustavo...@kernel.org>
---
 drivers/crypto/nx/nx-842.c |  6 ++++--
 drivers/crypto/nx/nx-842.h | 11 +++++++----
 2 files changed, 11 insertions(+), 6 deletions(-)

diff --git a/drivers/crypto/nx/nx-842.c b/drivers/crypto/nx/nx-842.c
index 2ab90ec10e61..82214cde2bcd 100644
--- a/drivers/crypto/nx/nx-842.c
+++ b/drivers/crypto/nx/nx-842.c
@@ -251,7 +251,9 @@ int nx842_crypto_compress(struct crypto_tfm *tfm,
                          u8 *dst, unsigned int *dlen)
 {
        struct nx842_crypto_ctx *ctx = crypto_tfm_ctx(tfm);
-       struct nx842_crypto_header *hdr = &ctx->header;
+       struct nx842_crypto_header *hdr =
+                               container_of(&ctx->header,
+                                            struct nx842_crypto_header, hdr);
        struct nx842_crypto_param p;
        struct nx842_constraints c = *ctx->driver->constraints;
        unsigned int groups, hdrsize, h;
@@ -490,7 +492,7 @@ int nx842_crypto_decompress(struct crypto_tfm *tfm,
        }
 
        memcpy(&ctx->header, src, hdr_len);
-       hdr = &ctx->header;
+       hdr = container_of(&ctx->header, struct nx842_crypto_header, hdr);
 
        for (n = 0; n < hdr->groups; n++) {
                /* ignore applies to last group */
diff --git a/drivers/crypto/nx/nx-842.h b/drivers/crypto/nx/nx-842.h
index 7590bfb24d79..1f42c83d2683 100644
--- a/drivers/crypto/nx/nx-842.h
+++ b/drivers/crypto/nx/nx-842.h
@@ -157,9 +157,12 @@ struct nx842_crypto_header_group {
 } __packed;
 
 struct nx842_crypto_header {
-       __be16 magic;           /* NX842_CRYPTO_MAGIC */
-       __be16 ignore;          /* decompressed end bytes to ignore */
-       u8 groups;              /* total groups in this header */
+       /* New members must be added within the __struct_group() macro below. */
+       __struct_group(nx842_crypto_header_hdr, hdr, __packed,
+               __be16 magic;           /* NX842_CRYPTO_MAGIC */
+               __be16 ignore;          /* decompressed end bytes to ignore */
+               u8 groups;              /* total groups in this header */
+       );
        struct nx842_crypto_header_group group[];
 } __packed;
 
@@ -171,7 +174,7 @@ struct nx842_crypto_ctx {
        u8 *wmem;
        u8 *sbounce, *dbounce;
 
-       struct nx842_crypto_header header;
+       struct nx842_crypto_header_hdr header;
        struct nx842_crypto_header_group group[NX842_CRYPTO_GROUP_MAX];
 
        struct nx842_driver *driver;
-- 
2.34.1

Reply via email to